package pwc.taxtech.atms.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.ebsdto.*;
import pwc.taxtech.atms.service.EbsApiService;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping("/ebs/api/v1/dd")
public class EbsApiController {

    private static final Logger logger = LoggerFactory.getLogger(EbsApiController.class);
    @Resource
    private EbsApiService ebsApiService;

    @RequestMapping(value = "/queryRemoteServerThenUpdateJE", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdateJE(@RequestBody List<JournalEntryQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdateJE return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateJE(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdateJE error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdateTB", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdateTB(@RequestBody List<TrialBalanceQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdateTB return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateTB(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdateTB error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdateCF", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdateCF(@RequestBody List<CashFlowQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdateCF return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateCF(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdateCF error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdateBS", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdateBS(@RequestBody List<BalanceSheetQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdateBS return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateBS(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdateBS error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdatePL", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdatePL(@RequestBody List<ProfitLossStatementQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdatePL return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdatePL(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdatePL error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdateBSprc", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdateBSprc(@RequestBody List<BalanceSheetPrcQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdateBSprc return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateBSprc(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdateBSprc error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdatePLprc", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdatePLprc(@RequestBody List<ProfitLossStatementPrcQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdatePL return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdatePLprc(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdatePLprc error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/queryRemoteServerThenUpdateOAR", method = RequestMethod.POST)
    public ApiResultDto queryRemoteServerThenUpdateOAR(@RequestBody List<OrganizationAccountingRateQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the queryRemoteServerThenUpdateOAR return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateOAR(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("queryRemoteServerThenUpdateOAR error.", e);
        }
        return ApiResultDto.fail();
    }

}
